<?php

declare(strict_types=1);

namespace Drupal\ui_patterns;

/**
 * Trait for plugins (sources and prop types) handling enum values.
 */
trait EnumTrait {

  /**
   * Get form element options from enumeration.
   */
  protected static function getEnumOptions(array $definition): array {
    $values = array_combine($definition['enum'], $definition['enum']);
    foreach ($values as $key => $label) {
      if (is_string($label)) {
        $values[$key] = ucwords($label);
      }
    }
    if (!isset($definition['meta:enum'])) {
      return array_values($values);
    }
    $meta = $definition['meta:enum'];
    // Remove meta:enum items not found in options.
    $meta = array_intersect_key($meta, $values);
    foreach ($meta as $value => $label) {
      $values[$value] = $label;
    }
    return $values;
  }

  /**
   * Get allowed values from enumeration.
   */
  protected static function getAllowedValues(array $definition): array {
    return array_values(static::getEnumOptions($definition));
  }

  /**
   * Converts a source value type to enum data type.
   *
   * @param mixed $value
   *   The stored.
   * @param array $enum
   *   The defined enums.
   *
   * @return float|int|mixed
   *   The converted value.
   */
  protected static function convertValueToEnumType(mixed $value, array $enum) {
    return match (TRUE) {
      in_array($value, $enum, TRUE) => $value,
      in_array((string) $value, $enum, TRUE) => (string) $value,
      in_array((int) $value, $enum, TRUE)  => (int) $value,
      in_array((float) $value, $enum, TRUE) => (float) $value,
      default => $value,
    };
  }

}
